import _extends from "@babel/runtime/helpers/esm/extends";
import { useState, useEffect, useRef, useCallback, cloneElement } from 'react';
import cx from 'classnames';
import PropTypes from 'prop-types';
import Button, { atomButtonDesigns } from '@s-ui/react-atom-button';
import { BASE_CLASS, getPlacement, PLACEMENTS, SIZES } from './config.js';
import { jsx as _jsx } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";

function usePrevious(value) {
  var ref = useRef();
  useEffect(function () {
    ref.current = value;
  }, [value]);
  return ref.current;
}

var popoverBaseClass = BASE_CLASS + "-popover";

function MoleculeSelectPopover(_ref) {
  var acceptButtonText = _ref.acceptButtonText,
      acceptButtonOptions = _ref.acceptButtonOptions,
      cancelButtonText = _ref.cancelButtonText,
      cancelButtonOptions = _ref.cancelButtonOptions,
      customButtonText = _ref.customButtonText,
      customButtonOptions = _ref.customButtonOptions,
      children = _ref.children,
      fullWidth = _ref.fullWidth,
      hideActions = _ref.hideActions,
      IconArrowDown = _ref.iconArrowDown,
      _ref$isDisabled = _ref.isDisabled,
      isDisabled = _ref$isDisabled === void 0 ? false : _ref$isDisabled,
      _ref$isSelected = _ref.isSelected,
      isSelected = _ref$isSelected === void 0 ? false : _ref$isSelected,
      _ref$onAccept = _ref.onAccept,
      onAccept = _ref$onAccept === void 0 ? function () {} : _ref$onAccept,
      _ref$onCancel = _ref.onCancel,
      onCancel = _ref$onCancel === void 0 ? function () {} : _ref$onCancel,
      _ref$onCancelButtonCl = _ref.onCancelButtonClick,
      onCancelButtonClick = _ref$onCancelButtonCl === void 0 ? function () {} : _ref$onCancelButtonCl,
      _ref$onClose = _ref.onClose,
      onClose = _ref$onClose === void 0 ? function () {} : _ref$onClose,
      _ref$onCustomAction = _ref.onCustomAction,
      onCustomAction = _ref$onCustomAction === void 0 ? function () {} : _ref$onCustomAction,
      _ref$onOpen = _ref.onOpen,
      onOpen = _ref$onOpen === void 0 ? function () {} : _ref$onOpen,
      placement = _ref.placement,
      renderContentWrapperProp = _ref.renderContentWrapper,
      renderSelectProp = _ref.renderSelect,
      selectText = _ref.selectText,
      _ref$size = _ref.size,
      size = _ref$size === void 0 ? 'm' : _ref$size,
      title = _ref.title;

  var _useState = useState(false),
      isOpen = _useState[0],
      setIsOpen = _useState[1];

  var _useState2 = useState(cx("" + popoverBaseClass, popoverBaseClass + "--" + getPlacement(placement))),
      popoverClassName = _useState2[0],
      setPopoverClassName = _useState2[1];

  var previousIsOpen = usePrevious(isOpen);
  var selectRef = useRef();
  var contentWrapperRef = useRef();
  useEffect(function () {
    /**
     * Only run open events:
     *  - After first render
     *  - When isOpen actually changes
     **/
    if (typeof previousIsOpen === 'undefined' || isOpen === previousIsOpen) {
      return;
    }

    if (isOpen && [PLACEMENTS.AUTO_END, PLACEMENTS.AUTO_START].includes(placement)) {
      setPopoverClassName(getPopoverClassName());
    }

    var openEvent = isOpen ? onOpen : onClose;
    openEvent();
  }, [isOpen, onClose, onOpen, previousIsOpen]);
  useEffect(function () {
    setPopoverClassName(cx("" + popoverBaseClass, popoverBaseClass + "--" + getPlacement(placement)));
  }, [placement]);

  var getPopoverClassName = function getPopoverClassName() {
    var _contentWrapperRef$cu2;

    var _contentWrapperRef$cu = (_contentWrapperRef$cu2 = contentWrapperRef.current) == null ? void 0 : _contentWrapperRef$cu2.getBoundingClientRect(),
        left = _contentWrapperRef$cu.left,
        right = _contentWrapperRef$cu.right;

    var outFromTheLeftSide = left < 0;
    var outFromTheRightSide = right > (window.innerWidth || document.documentElement.clientWidth);

    if (outFromTheRightSide) {
      return cx("" + popoverBaseClass, popoverBaseClass + "--left");
    } else if (outFromTheLeftSide) {
      return cx("" + popoverBaseClass, popoverBaseClass + "--right");
    }

    return popoverClassName;
  };

  var selectClassName = cx(BASE_CLASS + "-select", BASE_CLASS + "-select--" + size, {
    'is-open': isOpen,
    'is-selected': isSelected
  });

  var handleOnAccept = function handleOnAccept() {
    setIsOpen(false);
    onAccept();
  };

  var handleOnCustomAction = function handleOnCustomAction() {
    onCustomAction();
  };

  var handleCancelButtonClick = function handleCancelButtonClick() {
    onCancelButtonClick();
    handleOnCancel();
  };

  var handleOnCancel = useCallback(function () {
    setIsOpen(false);
    onCancel();
  }, [onCancel]);
  useEffect(function () {
    var handleClickOutside = function handleClickOutside(e) {
      if (isOpen && contentWrapperRef.current && !contentWrapperRef.current.contains(e.target) && !selectRef.current.contains(e.target)) {
        handleOnCancel();
      }
    };

    window.addEventListener('mousedown', handleClickOutside);
    return function () {
      window.removeEventListener('mousedown', handleClickOutside);
    };
  }, [handleOnCancel, isOpen]);

  var handleOpenToggle = function handleOpenToggle() {
    if (isOpen) {
      setIsOpen(false);
      handleOnCancel();
      return;
    }

    setIsOpen(true);
  };

  var renderActions = function renderActions() {
    return /*#__PURE__*/_jsxs(_Fragment, {
      children: [customButtonText ? /*#__PURE__*/_jsx(Button, _extends({
        onClick: handleOnCustomAction
      }, customButtonOptions, {
        children: customButtonText
      })) : null, cancelButtonText ? /*#__PURE__*/_jsx(Button, _extends({
        onClick: handleCancelButtonClick,
        design: atomButtonDesigns.FLAT
      }, cancelButtonOptions, {
        children: cancelButtonText
      })) : null, acceptButtonText ? /*#__PURE__*/_jsx(Button, _extends({
        onClick: handleOnAccept
      }, acceptButtonOptions, {
        children: acceptButtonText
      })) : null]
    });
  };

  var renderProp = function renderProp(render, props) {
    return typeof render === 'function' ? render(props) : /*#__PURE__*/cloneElement(render, _extends({}, render.props, props));
  };

  var renderSelect = function renderSelect() {
    var newSelectProps = {
      ref: selectRef,
      className: selectClassName,
      onClick: handleOpenToggle
    };

    if (renderSelectProp) {
      var _renderSelectProp$pro;

      var newRenderSelectProps = _extends({}, newSelectProps, {
        isOpen: isOpen,
        onClick: (_renderSelectProp$pro = renderSelectProp.props) != null && _renderSelectProp$pro.onClick ? function (ev) {
          renderSelectProp.props.onClick(ev);
          handleOpenToggle(ev);
        } : handleOpenToggle
      });

      return renderProp(renderSelectProp, newRenderSelectProps);
    }

    return /*#__PURE__*/_jsxs("div", _extends({
      className: selectClassName
    }, newSelectProps, {
      children: [/*#__PURE__*/_jsx("span", {
        className: BASE_CLASS + "-selectText",
        children: selectText
      }), /*#__PURE__*/_jsx("div", {
        className: BASE_CLASS + "-selectIcon",
        children: /*#__PURE__*/_jsx(IconArrowDown, {})
      })]
    }));
  };

  var renderContentWrapper = function renderContentWrapper() {
    var pieces = {
      actions: renderActions(),
      content: children,
      contentWrapperRef: contentWrapperRef,
      // required for `handleClickOutside` to work
      isOpen: isOpen,
      setIsOpen: setIsOpen
    };
    /**
     * Custom content wrapper from render prop
     */

    if (renderContentWrapperProp) {
      return renderProp(renderContentWrapperProp, pieces);
    }
    /**
     * Default content wrapper as a popover
     */


    return pieces.isOpen && /*#__PURE__*/_jsxs("div", {
      className: popoverClassName,
      ref: contentWrapperRef,
      children: [/*#__PURE__*/_jsx("div", {
        className: BASE_CLASS + "-popoverContent",
        children: pieces.content
      }), !hideActions && /*#__PURE__*/_jsx("div", {
        className: BASE_CLASS + "-popoverActionBar",
        children: pieces.actions
      })]
    });
  };

  var classNames = cx(BASE_CLASS, fullWidth && BASE_CLASS + "--fullWidth", isDisabled && 'is-disabled', renderContentWrapperProp && BASE_CLASS + "--hasCustomWrapper");
  return /*#__PURE__*/_jsxs("div", {
    className: classNames,
    title: title,
    children: [renderSelect(), renderContentWrapper()]
  });
}

MoleculeSelectPopover.displayName = 'MoleculeSelectPopover';
export default MoleculeSelectPopover;
export { SIZES as selectPopoverSizes, PLACEMENTS as selectPopoverPlacements };